@@ -21,16 +21,20 @@
 #include "config.h"
 #include "capabilities.h"
 
+#include <telepathy-glib/interfaces.h>
+#include <telepathy-glib/channel-manager.h>
+
+#include "caps-channel-manager.h"
 #include "namespaces.h"
 #include "presence-cache.h"
-#include <telepathy-glib/interfaces.h>
 #include "media-channel.h"
 
 static const Feature self_advertised_features[] =
 {
   { FEATURE_FIXED, NS_GOOGLE_FEAT_SESSION, 0},
   { FEATURE_FIXED, NS_GOOGLE_TRANSPORT_P2P, PRESENCE_CAP_GOOGLE_TRANSPORT_P2P},
-  { FEATURE_FIXED, NS_JINGLE, PRESENCE_CAP_JINGLE},
+  { FEATURE_FIXED, NS_JINGLE015, PRESENCE_CAP_JINGLE015},
+  { FEATURE_FIXED, NS_JINGLE032, PRESENCE_CAP_JINGLE032},
   { FEATURE_FIXED, NS_CHAT_STATES, PRESENCE_CAP_CHAT_STATES},
   { FEATURE_FIXED, NS_NICK, 0},
   { FEATURE_FIXED, NS_NICK "+notify", 0},
@@ -43,6 +47,7 @@ static const Feature self_advertised_features[] =
     PRESENCE_CAP_JINGLE_DESCRIPTION_AUDIO},
   { FEATURE_OPTIONAL, NS_JINGLE_DESCRIPTION_VIDEO,
     PRESENCE_CAP_JINGLE_DESCRIPTION_VIDEO},
+  { FEATURE_OPTIONAL, NS_JINGLE_RTP, PRESENCE_CAP_JINGLE_RTP},
 
   { FEATURE_OPTIONAL, NS_OLPC_BUDDY_PROPS "+notify", PRESENCE_CAP_OLPC_1},
   { FEATURE_OPTIONAL, NS_OLPC_ACTIVITIES "+notify", PRESENCE_CAP_OLPC_1},
@@ -53,8 +58,10 @@ static const Feature self_advertised_features[] =
 };
 
 GSList *
-capabilities_get_features (GabblePresenceCapabilities caps)
+capabilities_get_features (GabblePresenceCapabilities caps,
+                           GHashTable *per_channel_manager_caps)
 {
+  GHashTableIter channel_manager_iter;
   GSList *features = NULL;
   const Feature *i;
 
@@ -62,6 +69,20 @@ capabilities_get_features (GabblePresenceCapabilities caps)
     if ((i->caps & caps) == i->caps)
       features = g_slist_append (features, (gpointer) i);
 
+  if (per_channel_manager_caps != NULL)
+    {
+      gpointer manager;
+      gpointer cap;
+
+      g_hash_table_iter_init (&channel_manager_iter, per_channel_manager_caps);
+      while (g_hash_table_iter_next (&channel_manager_iter,
+                 &manager, &cap))
+        {
+          gabble_caps_channel_manager_get_feature_list (manager, cap,
+              &features);
+        }
+    }
+
   return features;
 }
 
