@@ -44,7 +44,7 @@ struct _Feature
     FEATURE_OPTIONAL,
     FEATURE_BUNDLE_COMPAT   /* just for voice-v1 */
   } feature_type;
-  const gchar *ns;
+  gchar *ns;
   GabblePresenceCapabilities caps;
 };
 
@@ -54,7 +54,8 @@ struct _Feature
  * Return a linked list of const Feature structs corresponding to the given
  * GabblePresenceCapabilities.
  */
-GSList *capabilities_get_features (GabblePresenceCapabilities caps);
+GSList *capabilities_get_features (GabblePresenceCapabilities caps,
+    GHashTable *per_channel_manager_caps);
 
 /*
  * capabilities_fill_cache
