@@ -21,6 +21,7 @@
 #include "private-tubes-factory.h"
 
 #include <string.h>
+#include <glib-object.h>
 
 #include <dbus/dbus-glib.h>
 #include <dbus/dbus-glib-lowlevel.h>
@@ -35,16 +36,20 @@
 
 #define DEBUG_FLAG GABBLE_DEBUG_TUBES
 
+#include "caps-channel-manager.h"
+#include "capabilities.h"
 #include "connection.h"
 #include "debug.h"
 #include "muc-channel.h"
 #include "muc-factory.h"
 #include "namespaces.h"
+#include "presence-cache.h"
 #include "tubes-channel.h"
 #include "util.h"
 
 static GabbleTubesChannel *new_tubes_channel (GabblePrivateTubesFactory *fac,
-    TpHandle handle, TpHandle initiator, gpointer request_token);
+    TpHandle handle, TpHandle initiator, gpointer request_token,
+    gboolean send_new_channel_signal);
 
 static void tubes_channel_closed_cb (GabbleTubesChannel *chan,
     gpointer user_data);
@@ -54,12 +59,15 @@ static LmHandlerResult private_tubes_factory_msg_tube_cb (
     gpointer user_data);
 
 static void channel_manager_iface_init (gpointer, gpointer);
+static void caps_channel_manager_iface_init (gpointer, gpointer);
 
 G_DEFINE_TYPE_WITH_CODE (GabblePrivateTubesFactory,
     gabble_private_tubes_factory,
     G_TYPE_OBJECT,
     G_IMPLEMENT_INTERFACE (TP_TYPE_CHANNEL_MANAGER,
-      channel_manager_iface_init));
+      channel_manager_iface_init);
+    G_IMPLEMENT_INTERFACE (GABBLE_TYPE_CAPS_CHANNEL_MANAGER,
+      caps_channel_manager_iface_init));
 
 /* properties */
 enum
@@ -81,6 +89,36 @@ struct _GabblePrivateTubesFactoryPrivate
 
 #define GABBLE_PRIVATE_TUBES_FACTORY_GET_PRIVATE(obj) ((obj)->priv)
 
+typedef struct _TubesCapabilities TubesCapabilities;
+struct _TubesCapabilities
+{
+  /* Stores the list of tubes supported by a contact. We use a hash table. The
+   * key is the service name and the value is NULL.
+   *
+   * It can also be used to store the list of tubes that Gabble advertises to
+   * support when Gabble replies to XEP-0115 Entity Capabilities requests. In
+   * this case, a Feature structure is associated with each tube type in order
+   * to be returned by gabble_private_tubes_factory_get_feature_list().
+   *
+   * So the value of the hash table is either NULL (if the variable is related
+   * to a contact handle), or a Feature structure (if the variable is related
+   * to the self_handle).
+   */
+
+  /* gchar *Service -> NULL
+   *  or
+   * gchar *Service -> Feature *feature
+   */
+  GHashTable *stream_tube_caps;
+
+  /* gchar *ServiceName -> NULL
+   *  or
+   * gchar *ServiceName -> Feature *feature
+   */
+  GHashTable *dbus_tube_caps;
+};
+
+
 static void
 gabble_private_tubes_factory_init (GabblePrivateTubesFactory *self)
 {
@@ -227,7 +265,7 @@ gabble_private_tubes_factory_class_init (
   param_spec = g_param_spec_object (
       "connection",
       "GabbleConnection object",
-      "Gabble connection object that owns this Tubes channel factory object.",
+      "Gabble connection object that owns this Tubes channel manager object.",
       GABBLE_TYPE_CONNECTION,
       G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
   g_object_class_install_property (object_class, PROP_CONNECTION, param_spec);
@@ -272,13 +310,13 @@ static GabbleTubesChannel *
 new_tubes_channel (GabblePrivateTubesFactory *fac,
                    TpHandle handle,
                    TpHandle initiator,
-                   gpointer request_token)
+                   gpointer request_token,
+                   gboolean send_new_channel_signal)
 {
   GabblePrivateTubesFactoryPrivate *priv;
   TpBaseConnection *conn;
   GabbleTubesChannel *chan;
   char *object_path;
-  GSList *request_tokens;
 
   g_assert (GABBLE_IS_PRIVATE_TUBES_FACTORY (fac));
   g_assert (handle != 0);
@@ -306,15 +344,19 @@ new_tubes_channel (GabblePrivateTubesFactory *fac,
 
   g_free (object_path);
 
-  if (request_token != NULL)
-    request_tokens = g_slist_prepend (NULL, request_token);
-  else
-    request_tokens = NULL;
+  if (send_new_channel_signal)
+    {
+      GSList *request_tokens;
+      if (request_token != NULL)
+        request_tokens = g_slist_prepend (NULL, request_token);
+      else
+        request_tokens = NULL;
 
-  tp_channel_manager_emit_new_channel (fac,
-      TP_EXPORTABLE_CHANNEL (chan), request_tokens);
+      tp_channel_manager_emit_new_channel (fac,
+          TP_EXPORTABLE_CHANNEL (chan), request_tokens);
 
-  g_slist_free (request_tokens);
+      g_slist_free (request_tokens);
+    }
 
   return chan;
 }
@@ -342,6 +384,9 @@ gabble_private_tubes_factory_close_all (GabblePrivateTubesFactory *fac)
       priv->msg_tube_cb = NULL;
     }
 
+  /* Use a temporary variable because we don't want
+   * tubes_channel_closed_cb to remove the channel from the hash table a
+   * second time */
   if (priv->channels != NULL)
     {
       GHashTable *tmp = priv->channels;
@@ -351,6 +396,411 @@ gabble_private_tubes_factory_close_all (GabblePrivateTubesFactory *fac)
     }
 }
 
+static void
+add_service_to_array (gchar *service,
+                      GPtrArray *arr,
+                      TpTubeType type,
+                      TpHandle handle)
+{
+  GValue monster = {0, };
+  GHashTable *fixed_properties;
+  GValue *channel_type_value;
+  GValue *target_handle_type_value;
+  gchar *tube_allowed_properties[] =
+      {
+        TP_IFACE_CHANNEL ".TargetHandle",
+        NULL
+      };
+
+  g_assert (type == TP_TUBE_TYPE_STREAM || type == TP_TUBE_TYPE_DBUS);
+
+  g_value_init (&monster, TP_STRUCT_TYPE_REQUESTABLE_CHANNEL_CLASS);
+  g_value_take_boxed (&monster,
+      dbus_g_type_specialized_construct (
+        TP_STRUCT_TYPE_REQUESTABLE_CHANNEL_CLASS));
+
+  fixed_properties = g_hash_table_new_full (g_str_hash, g_str_equal, NULL,
+      (GDestroyNotify) tp_g_value_slice_free);
+
+  channel_type_value = tp_g_value_slice_new (G_TYPE_STRING);
+  if (type == TP_TUBE_TYPE_STREAM)
+    g_value_set_static_string (channel_type_value,
+        GABBLE_IFACE_CHANNEL_TYPE_STREAM_TUBE);
+  else
+    g_value_set_static_string (channel_type_value,
+        GABBLE_IFACE_CHANNEL_TYPE_DBUS_TUBE);
+  g_hash_table_insert (fixed_properties, TP_IFACE_CHANNEL ".ChannelType",
+      channel_type_value);
+
+  target_handle_type_value = tp_g_value_slice_new (G_TYPE_UINT);
+  g_value_set_uint (target_handle_type_value, TP_HANDLE_TYPE_CONTACT);
+  g_hash_table_insert (fixed_properties,
+      TP_IFACE_CHANNEL ".TargetHandleType", target_handle_type_value);
+
+  target_handle_type_value = tp_g_value_slice_new (G_TYPE_STRING);
+  g_value_set_string (target_handle_type_value, service);
+  if (type == TP_TUBE_TYPE_STREAM)
+    g_hash_table_insert (fixed_properties,
+        GABBLE_IFACE_CHANNEL_TYPE_STREAM_TUBE ".Service",
+        target_handle_type_value);
+  else
+    g_hash_table_insert (fixed_properties,
+        GABBLE_IFACE_CHANNEL_TYPE_DBUS_TUBE ".ServiceName",
+        target_handle_type_value);
+
+  dbus_g_type_struct_set (&monster,
+      0, fixed_properties,
+      1, tube_allowed_properties,
+      G_MAXUINT);
+
+  g_hash_table_destroy (fixed_properties);
+
+  g_ptr_array_add (arr, g_value_get_boxed (&monster));
+}
+
+static void
+gabble_private_tubes_factory_get_contact_caps (
+    GabbleCapsChannelManager *manager,
+    GabbleConnection *conn,
+    TpHandle handle,
+    GPtrArray *arr)
+{
+  TpBaseConnection *base = (TpBaseConnection *) conn;
+  TubesCapabilities *caps;
+  GHashTable *stream_tube_caps;
+  GHashTable *dbus_tube_caps;
+  GabblePresence *presence;
+  GHashTableIter tube_caps_iter;
+  gpointer service;
+
+  g_assert (handle != 0);
+
+  if (handle == base->self_handle)
+    presence = conn->self_presence;
+  else
+    presence = gabble_presence_cache_get (conn->presence_cache, handle);
+
+  if (presence == NULL)
+    return;
+
+  if (presence->per_channel_manager_caps == NULL)
+    return;
+
+  caps = g_hash_table_lookup (presence->per_channel_manager_caps, manager);
+  if (caps == NULL)
+    return;
+
+  stream_tube_caps = caps->stream_tube_caps;
+  dbus_tube_caps = caps->dbus_tube_caps;
+
+  if (stream_tube_caps != NULL)
+    {
+      g_hash_table_iter_init (&tube_caps_iter, stream_tube_caps);
+      while (g_hash_table_iter_next (&tube_caps_iter, &service,
+            NULL))
+        {
+          add_service_to_array (service, arr, TP_TUBE_TYPE_STREAM, handle);
+        }
+    }
+
+  if (dbus_tube_caps != NULL)
+    {
+      g_hash_table_iter_init (&tube_caps_iter, dbus_tube_caps);
+      while (g_hash_table_iter_next (&tube_caps_iter, &service,
+            NULL))
+        {
+          add_service_to_array (service, arr, TP_TUBE_TYPE_DBUS, handle);
+        }
+    }
+}
+
+static void
+gabble_private_tubes_factory_get_feature_list (
+    GabbleCapsChannelManager *manager,
+    gpointer specific_caps,
+    GSList **features)
+{
+  TubesCapabilities *caps = specific_caps;
+  GHashTableIter iter;
+  gpointer service;
+  gpointer feat;
+
+  g_hash_table_iter_init (&iter, caps->stream_tube_caps);
+  while (g_hash_table_iter_next (&iter, &service,
+        &feat))
+    {
+      *features = g_slist_append (*features, feat);
+    }
+
+  g_hash_table_iter_init (&iter, caps->dbus_tube_caps);
+  while (g_hash_table_iter_next (&iter, &service,
+        &feat))
+    {
+      *features = g_slist_append (*features, feat);
+    }
+}
+
+static void
+gabble_private_tubes_factory_free_feat (gpointer data)
+{
+  Feature *feat = (Feature *) data;
+
+  if (feat == NULL)
+    return;
+
+  if (feat->ns != NULL)
+    g_free (feat->ns);
+
+  g_free (feat);
+}
+
+static gpointer
+gabble_private_tubes_factory_parse_caps (
+    GabbleCapsChannelManager *manager,
+    LmMessageNode *children)
+{
+  LmMessageNode *child;
+  TubesCapabilities *caps;
+
+  caps = g_new0 (TubesCapabilities, 1);
+  caps->stream_tube_caps = g_hash_table_new_full (g_str_hash, g_str_equal,
+      g_free, gabble_private_tubes_factory_free_feat);
+  caps->dbus_tube_caps = g_hash_table_new_full (g_str_hash, g_str_equal,
+      g_free, gabble_private_tubes_factory_free_feat);
+
+  for (child = children; NULL != child; child = child->next)
+    {
+      const gchar *var;
+
+      if (0 != strcmp (child->name, "feature"))
+        continue;
+
+      var = lm_message_node_get_attribute (child, "var");
+
+      if (NULL == var)
+        continue;
+
+      if (g_str_has_prefix (var, NS_TUBES "/"))
+        {
+          /* http://telepathy.freedesktop.org/xmpp/tubes/$type#$service */
+          var += strlen (NS_TUBES "/");
+          if (g_str_has_prefix (var, "stream#"))
+            {
+              gchar *service;
+              var += strlen ("stream#");
+              service = g_strdup (var);
+              g_hash_table_insert (caps->stream_tube_caps, service, NULL);
+            }
+          else if (g_str_has_prefix (var, "dbus#"))
+            {
+              gchar *service;
+              var += strlen ("dbus#");
+              service = g_strdup (var);
+              g_hash_table_insert (caps->dbus_tube_caps, service, NULL);
+            }
+        }
+    }
+
+  return caps;
+}
+
+static void
+gabble_private_tubes_factory_free_caps (
+    GabbleCapsChannelManager *manager,
+    gpointer data)
+{
+ TubesCapabilities *caps = data;
+ g_hash_table_destroy (caps->stream_tube_caps);
+ g_hash_table_destroy (caps->dbus_tube_caps);
+ g_free (caps);
+}
+
+static void
+copy_caps_helper (gpointer key, gpointer value, gpointer user_data)
+{
+  GHashTable *out = user_data;
+  gchar *str = key;
+
+  g_hash_table_insert (out, g_strdup (str), NULL);
+}
+
+static void
+gabble_private_tubes_factory_copy_caps (
+    GabbleCapsChannelManager *manager,
+    gpointer *specific_caps_out,
+    gpointer specific_caps_in)
+{
+  TubesCapabilities *caps_in = specific_caps_in;
+  TubesCapabilities *caps_out = g_new0 (TubesCapabilities, 1);
+
+  caps_out->stream_tube_caps = g_hash_table_new_full (g_str_hash, g_str_equal,
+      g_free, gabble_private_tubes_factory_free_feat);
+  g_hash_table_foreach (caps_in->stream_tube_caps, copy_caps_helper,
+      caps_out->stream_tube_caps);
+
+  caps_out->dbus_tube_caps = g_hash_table_new_full (g_str_hash, g_str_equal,
+      g_free, gabble_private_tubes_factory_free_feat);
+  g_hash_table_foreach (caps_in->dbus_tube_caps, copy_caps_helper,
+      caps_out->dbus_tube_caps);
+
+  *specific_caps_out = caps_out;
+}
+
+static void
+gabble_private_tubes_factory_update_caps (
+    GabbleCapsChannelManager *manager,
+    gpointer specific_caps_out,
+    gpointer specific_caps_in)
+{
+  TubesCapabilities *caps_out = (TubesCapabilities *) specific_caps_out;
+  TubesCapabilities *caps_in = (TubesCapabilities *) specific_caps_in;
+
+  if (caps_in == NULL)
+    return;
+
+  tp_g_hash_table_update (caps_out->stream_tube_caps,
+      caps_in->stream_tube_caps, (GBoxedCopyFunc) g_strdup, NULL);
+  tp_g_hash_table_update (caps_out->dbus_tube_caps,
+      caps_in->dbus_tube_caps, (GBoxedCopyFunc) g_strdup, NULL);
+}
+
+static gboolean
+gabble_private_tubes_factory_caps_diff (
+    GabbleCapsChannelManager *manager,
+    TpHandle handle,
+    gpointer specific_old_caps,
+    gpointer specific_new_caps)
+{
+  TubesCapabilities *old_caps = specific_old_caps;
+  TubesCapabilities *new_caps = specific_new_caps;
+  GHashTableIter tube_caps_iter;
+  gpointer service;
+
+  if (old_caps != NULL)
+    {
+      g_hash_table_iter_init (&tube_caps_iter, old_caps->stream_tube_caps);
+      while (g_hash_table_iter_next (&tube_caps_iter, &service,
+            NULL))
+        {
+          gpointer key, value;
+          if (new_caps == NULL ||
+              !g_hash_table_lookup_extended (new_caps->stream_tube_caps,
+                  service, &key, &value))
+            {
+              return TRUE;
+            }
+        }
+      g_hash_table_iter_init (&tube_caps_iter, old_caps->dbus_tube_caps);
+      while (g_hash_table_iter_next (&tube_caps_iter, &service,
+            NULL))
+        {
+          gpointer key, value;
+          if (new_caps == NULL ||
+              !g_hash_table_lookup_extended (new_caps->dbus_tube_caps,
+                  service, &key, &value))
+            {
+              return TRUE;
+            }
+        }
+    }
+
+  if (new_caps != NULL)
+    {
+      g_hash_table_iter_init (&tube_caps_iter, new_caps->stream_tube_caps);
+      while (g_hash_table_iter_next (&tube_caps_iter, &service,
+            NULL))
+        {
+          gpointer key, value;
+          if (old_caps == NULL ||
+              !g_hash_table_lookup_extended (old_caps->stream_tube_caps,
+                  service, &key, &value))
+            {
+              return TRUE;
+            }
+        }
+      g_hash_table_iter_init (&tube_caps_iter, new_caps->dbus_tube_caps);
+      while (g_hash_table_iter_next (&tube_caps_iter, &service,
+            NULL))
+        {
+          gpointer key, value;
+          if (old_caps == NULL ||
+              !g_hash_table_lookup_extended (old_caps->dbus_tube_caps,
+                  service, &key, &value))
+            {
+              return TRUE;
+            }
+        }
+    }
+  return FALSE;
+}
+
+static void
+gabble_private_tubes_factory_add_cap (GabbleCapsChannelManager *manager,
+                                      GabbleConnection *conn,
+                                      TpHandle handle,
+                                      GHashTable *cap)
+{
+  TpBaseConnection *base = (TpBaseConnection *) conn;
+  GabblePresence *presence;
+  TubesCapabilities *caps;
+  const gchar *channel_type;
+
+  channel_type = tp_asv_get_string (cap,
+            TP_IFACE_CHANNEL ".ChannelType");
+
+  /* this channel is not for this factory */
+  if (tp_strdiff (channel_type, TP_IFACE_CHANNEL_TYPE_TUBES) &&
+      tp_strdiff (channel_type, GABBLE_IFACE_CHANNEL_TYPE_STREAM_TUBE) &&
+      tp_strdiff (channel_type, GABBLE_IFACE_CHANNEL_TYPE_DBUS_TUBE))
+    return;
+
+  if (tp_asv_get_uint32 (cap,
+        TP_IFACE_CHANNEL ".TargetHandleType", NULL) != TP_HANDLE_TYPE_CONTACT)
+    return;
+
+  if (handle == base->self_handle)
+    presence = conn->self_presence;
+  else
+    presence = gabble_presence_cache_get (conn->presence_cache, handle);
+
+  g_assert (presence != NULL);
+
+  if (presence->per_channel_manager_caps == NULL)
+    presence->per_channel_manager_caps = g_hash_table_new (NULL, NULL);
+
+  caps = g_hash_table_lookup (presence->per_channel_manager_caps, manager);
+  if (caps == NULL)
+    {
+      caps = g_new0 (TubesCapabilities, 1);
+      caps->stream_tube_caps = g_hash_table_new_full (g_str_hash, g_str_equal,
+          g_free, gabble_private_tubes_factory_free_feat);
+      caps->dbus_tube_caps = g_hash_table_new_full (g_str_hash, g_str_equal,
+          g_free, gabble_private_tubes_factory_free_feat);
+      g_hash_table_insert (presence->per_channel_manager_caps, manager, caps);
+    }
+
+  if (!tp_strdiff (channel_type, GABBLE_IFACE_CHANNEL_TYPE_STREAM_TUBE))
+    {
+      Feature *feat = g_new0 (Feature, 1);
+      gchar *service = g_strdup (tp_asv_get_string (cap,
+          GABBLE_IFACE_CHANNEL_TYPE_STREAM_TUBE ".Service"));
+      feat->feature_type = FEATURE_OPTIONAL;
+      feat->ns = g_strdup_printf ("%s/stream#%s", NS_TUBES, service);
+      feat->caps = 0;
+      g_hash_table_insert (caps->stream_tube_caps, service, feat);
+    }
+  else if (!tp_strdiff (channel_type, GABBLE_IFACE_CHANNEL_TYPE_DBUS_TUBE))
+    {
+      Feature *feat = g_new0 (Feature, 1);
+      gchar *service = g_strdup (tp_asv_get_string (cap,
+          GABBLE_IFACE_CHANNEL_TYPE_DBUS_TUBE ".ServiceName"));
+      feat->feature_type = FEATURE_OPTIONAL;
+      feat->ns = g_strdup_printf ("%s/dbus#%s", NS_TUBES, service);
+      feat->caps = 0;
+      g_hash_table_insert (caps->dbus_tube_caps, service, feat);
+    }
+}
+
 struct _ForeachData
 {
   TpExportableChannelFunc foreach;
@@ -410,7 +860,7 @@ gabble_private_tubes_factory_handle_si_tube_request (
   chan = g_hash_table_lookup (priv->channels, GUINT_TO_POINTER (handle));
   if (chan == NULL)
     {
-      chan = new_tubes_channel (self, handle, handle, NULL);
+      chan = new_tubes_channel (self, handle, handle, NULL, TRUE);
 
       /* FIXME: Should we close the channel if the request is not properly
        * handled by the newly created channel ? */
@@ -497,7 +947,7 @@ private_tubes_factory_msg_tube_cb (LmMessageHandler *handler,
         {
           /* We create the tubes channel only if the message is a new tube
            * offer */
-          chan = new_tubes_channel (self, handle, handle, NULL);
+          chan = new_tubes_channel (self, handle, handle, NULL, TRUE);
         }
       else
         {
@@ -625,8 +1075,8 @@ gabble_private_tubes_factory_requestotron (GabblePrivateTubesFactory *self,
   TpBaseConnection *base_conn = (TpBaseConnection *) self->priv->conn;
   TpHandle handle;
   GError *error = NULL;
-  GabbleTubesChannel *channel;
   const gchar *channel_type;
+  GabbleTubesChannel *channel;
 
   if (tp_asv_get_uint32 (request_properties,
         TP_IFACE_CHANNEL ".TargetHandleType", NULL) != TP_HANDLE_TYPE_CONTACT)
@@ -664,7 +1114,7 @@ gabble_private_tubes_factory_requestotron (GabblePrivateTubesFactory *self,
       if (service == NULL)
         {
           g_set_error (&error, TP_ERRORS, TP_ERROR_NOT_IMPLEMENTED,
-              "Request missed a mandatory property '%s'",
+              "Request does not contain the mandatory property '%s'",
               GABBLE_IFACE_CHANNEL_TYPE_STREAM_TUBE ".Service");
           goto error;
         }
@@ -685,7 +1135,7 @@ gabble_private_tubes_factory_requestotron (GabblePrivateTubesFactory *self,
       if (service == NULL)
         {
           g_set_error (&error, TP_ERRORS, TP_ERROR_NOT_IMPLEMENTED,
-              "Request missed a mandatory property '%s'",
+              "Request does not contain the mandatory property '%s'",
               GABBLE_IFACE_CHANNEL_TYPE_DBUS_TUBE ".ServiceName");
           goto error;
         }
@@ -712,7 +1162,7 @@ gabble_private_tubes_factory_requestotron (GabblePrivateTubesFactory *self,
       if (channel == NULL)
         {
           channel = new_tubes_channel (self, handle, base_conn->self_handle,
-              request_token);
+              request_token, TRUE);
           return TRUE;
         }
 
@@ -731,7 +1181,7 @@ gabble_private_tubes_factory_requestotron (GabblePrivateTubesFactory *self,
     }
   else
     {
-      gboolean channel_was_existing = (channel != NULL);
+      gboolean channel_already_existed = (channel != NULL);
       GabbleTubeIface *new_channel;
 
       if (channel == NULL)
@@ -739,7 +1189,7 @@ gabble_private_tubes_factory_requestotron (GabblePrivateTubesFactory *self,
           /* Don't give the request_token to new_tubes_channel() because we
            * must emit NewChannels with 2 channels together */
           channel = new_tubes_channel (self, handle, base_conn->self_handle,
-              NULL);
+              NULL, FALSE);
         }
       g_assert (channel != NULL);
 
@@ -752,7 +1202,7 @@ gabble_private_tubes_factory_requestotron (GabblePrivateTubesFactory *self,
 
           channels = g_hash_table_new_full (g_direct_hash, g_direct_equal,
               NULL, NULL);
-          if (!channel_was_existing)
+          if (!channel_already_existed)
             g_hash_table_insert (channels, channel, NULL);
 
           if (request_token != NULL)
@@ -832,3 +1282,19 @@ channel_manager_iface_init (gpointer g_iface,
   iface->request_channel = gabble_private_tubes_factory_request_channel;
   iface->ensure_channel = gabble_private_tubes_factory_ensure_channel;
 }
+
+static void
+caps_channel_manager_iface_init (gpointer g_iface,
+                                 gpointer iface_data)
+{
+  GabbleCapsChannelManagerIface *iface = g_iface;
+
+  iface->get_contact_caps = gabble_private_tubes_factory_get_contact_caps;
+  iface->get_feature_list = gabble_private_tubes_factory_get_feature_list;
+  iface->parse_caps = gabble_private_tubes_factory_parse_caps;
+  iface->free_caps = gabble_private_tubes_factory_free_caps;
+  iface->copy_caps = gabble_private_tubes_factory_copy_caps;
+  iface->update_caps = gabble_private_tubes_factory_update_caps;
+  iface->caps_diff = gabble_private_tubes_factory_caps_diff;
+  iface->add_cap = gabble_private_tubes_factory_add_cap;
+}
