@@ -25,6 +25,7 @@
 #include <telepathy-glib/enums.h>
 
 #include "connection.h"
+#include "extensions/extensions.h"
 
 G_BEGIN_DECLS
 
@@ -34,6 +35,8 @@ typedef struct _GabbleTubeStreamClass GabbleTubeStreamClass;
 
 struct _GabbleTubeStreamClass {
   GObjectClass parent_class;
+
+  TpDBusPropertiesMixinClass dbus_props_class;
 };
 
 struct _GabbleTubeStream {
@@ -69,6 +72,12 @@ gboolean gabble_tube_stream_check_params (TpSocketAddressType address_type,
     const GValue *address, TpSocketAccessControl access_control,
     const GValue *access_control_param, GError **error);
 
+gboolean gabble_tube_stream_offer (GabbleTubeStream *self, guint address_type,
+    const GValue *address, guint access_control,
+    const GValue *access_control_param, GError **error);
+
+GHashTable *gabble_tube_stream_get_supported_socket_types (void);
+
 G_END_DECLS
 
 #endif /* #ifndef __GABBLE_TUBE_STREAM_H__ */
