@@ -95,7 +95,7 @@ main (void)
                                gabble_protocols, &error);
   if (!s)
     {
-      fprintf (stderr, error->message);
+      fprintf (stderr, "%s", error->message);
       g_error_free (error);
       return 1;
     }
